/**
 * The <code>Stack</code> class represents a last-in-first-out 
 * (LIFO) stack of objects. It extends class <tt>Vector</tt> with five 
 * operations that allow a vector to be treated as a stack. The usual 
 * <tt>push</tt> and <tt>pop</tt> operations are provided, as well as a
 * method to <tt>peek</tt> at the top item on the stack, a method to test 
 * for whether the stack is <tt>empty</tt>, and a method to <tt>search</tt> 
 * the stack for an item and discover how far it is from the top.
 * <p>
 * When a stack is first created, it contains no items.
 **/
public interface StackInterface {
/**
 * Determines whether the stack is empty.
 * <p>
 * Precondition: None.
 * <p>
 * Postcondition: Returns true if the stack is empty;
 * otherwise returns false.
 * <p>
 * @return <code>true</code> if the stack is empty, <code>false</code> otherwise.
 **/
  public boolean isEmpty();

/**
 * Removes all the items from the stack.
 * <p>
 * Precondition: None.
 * <p>
 * PostCondition: Stack is empty.
 */
  public void popAll();

/**
 * Adds an item to the top of a stack.
 * <p>
 * Precondition: newItem is the item to be added.
 * <p>
 * Postcondition: If insertion is successful, newItem
 * is on the top of the stack.
 * <p>
 * @param newItem any object
 * <p>
 * @throws Some implementations may throw 
 * StackException when newItem cannot be placed on
 * the stack.
 */
  public void push(Object newItem) throws StackException;

/**
 * Removes the top of a stack.
 * <p>
 * Precondition: None.
 * <p>
 * Postcondition: If the stack is not empty, the item 
 * that was added most recently is removed from the
 * stack and returned.
 * <p>
 * @return the object most recently added to the stack
 * <p>
 * @throws StackException if the stack is 
 * empty.
 */
  public Object pop() throws StackException;
  
/**
 * Retrieves the top of a stack.
 * <p>
 * Precondition: None.
 * <p>
 * Postcondition: If the stack is not empty, the item
 * that was added most recently is returned. The 
 * stack is unchanged. 
 * <p>
 * @param i the index of the element to be returned, counting from
 * the top of the stack
 * <p>
 * @return the object most recently added to the stack
 * <p>
 * @throws StackException if the stack is * empty.
 * @throws IllegalArgumentException if the index is < 0 
 * or >= the number of elements in the stack.
 */
  public Object peek(int i) throws StackException;
}  // end StackInterface
